
; Simple DSP Weaver TX.
;Uses Xtal oscillator. You must decouple Vcc & Core
;and bias ADC input to half supply. That's it. There's no agc on
;audio i/p as we only have 12bits so keep audio p-p swing to <3V
;Also note that while you can add/sub a DC offset to balance the
;external TX mixers this reduces the dynamic range and more importantly
;you have to add EEPROM to store it.
;Similary the gain variabilty of the DAC may mean external gain pot is reqd.
;This version puts more 4* gain in the audio paths if Weaver is selected as filter coeffs
;are a bit mean cf. the twisted phasing FIRs.

;Pulling PORTB,0 to gnd (Pin4) turns it into a phasing TX by connecting the
;Mic filter directly to the I/Q filters which are changed to +/- 45 BPFs
;LSB/USB select by swapping the DACs if you pull down pin5
;Pulling PORTB, 0 to gnd (Pin6) sends plain carrier.  A DC value is sent to I and Q DACs

;Mods for Xtal version.  Config bits changed for XT oscillator with PLL
; PLL divider changed for a wider SSB bandwidth and 1.8kHz centre Fs = 28800Hz

;Martin G8ASG.  Andy  G4JNT
;-----------------------------------------------
;Select at test constants added to DAC output (different for each unit)   

 ; I_OFFSETr	=	    0		; Uncommitted
 ; Q_OFFSETl	=	    0		;  " " " " "

 ; I_OFFSETr	=	-1800		; PCB No 1
 ; Q_OFFSETl	=	  530		;  " " " " 

 ; I_OFFSETr	=	-1110		;PCB No 2 S-Band upconv
 ; Q_OFFSETl	=	+1405		;  " " " " " " " " " "
 
  I_OFFSETr	=	    0		;PCB No 3  breadboard
  Q_OFFSETl	=	    0		;  " " " " " " " " "

 ; I_OFFSETr	=	3400		;PCB No 4 
 ; Q_OFFSETl	=	-1008		;  " " " "

 LEDGrn	= #7
 LEDRed	= #8
 NVOX 	= #9
 
 OvlFlag = #1		;Bit No. in Flags register
 VOXFlag = #2


  Carrier	=	8000	;Fixed value sent to A/D for plain carrier. No offsets, so don't expect equal DC for I/Q
  Dacdiv	=	4		;DAC final divider -1    Fsample = 1.44MHz / (Dacdiv + 1)
  MinThresh	=	512	;[12bit signed ADC] ^ 2 / 65536.     = [ADC reading ^2] / 256
  MaxThresh	=	15600  ; Bit less than ADCmax ^ 2 / 256
  OvlTimeout = 5000		;Clocks of 28800Hz for overload LED flash duration
  VOXDelay	 = 14400	;500ms
  AVNo		=	200
  VOXUpperTh = 100		;These are based on Rolling average of ADC^2 and are different from 
  VOXLowerTh = 20
;-----------------------------------------------
        .include "p33fj64gp802.inc"

; DSPIC33FJ64GP802 Configuration Bit Settings
config __FBS,    0xffff      
;Config words from Config bits dropdown
config __FSS,    0xffff      
;
config __FGS,    0xffff
;config __FOSCSEL,0x0001	;For RC oscillator	
config __FOSCSEL,0x0003		;Xtal osc
;Int osc,PLL
;config __FOSC,   0xffe3	;For RC Oscillator
config __FOSC,   0xffe1	
;Primary osc disable, no osc out
config __FWDT,   0x005f
config __FPOR,   0x00f7		
;SDA1 128mS PU
config __FICD,   0xff5d
;0xff5f ;PGC3
;..............................................................................


;Program Specific Constants (literals used in code)
;..............................................................................

;..............................................................................
;Global Declarations:
;On reset the reset vector table forces a jump to __reset
        		.global __reset          ;The label for the first line of code.
;DAC interrupt jump to here
				.global __DAC1RInterrupt
;				.global __MI2C1Interrupt	;I2C routine
;				.global __CNInterrupt		;Rotary VFO control
				.global __SP_init
;..............................................................................
;Uninitialized variables in X-space in data memory
;..............................................................................
         		.section .xbss, bss, xmemory
;All RAM that is not Y or DMA is XRAM
;..............................................................................
;Uninitialized variables in Y-space in data memory
          		.section .ybss, bss, ymemory 
/*These circular buffers use incrementing pointers so must start on XX00 boundries.
;As they are > 256 bytes long we must start at 512 bytes boundries (See modulo addressing)
*/
				.align 512				;<Important for operation of modulo addressing
I_Delay:  		.space 2*(201) 
I_Ptr:     		.space 2               	; I channel pointer to I_Filter
I_LPF_Out: 		.space 2				;I LPF output

				.align 512				;<Important for operation of modulo addressing
Q_Delay:  		.space 2*(201) 
Q_Ptr:     		.space 2               	;Q channel pointer to Q_Filter
Q_LPF_Out: 		.space 2				;Q LPF output

				.align 512				;<Important for operation of modulo addressing
Mic_Delay:  	.space 2*(201) 
Mic_ADC_Out:	.space 2				;Output sample from ADC
ADC_Out:		.space 2
Mic_Ptr:	   	.space 2               	; pointer to Mic_Filter
Mic_BPF_Out:   	.space 2 				; sample value from Mic BPFilter output

DiffGain:		.space 2
;..............................................................................
;Uninitialized variables in Near data memory (Lower 8Kb of RAM)
;..............................................................................
          		.section .nbss, bss, near

CycleCounter:	.space 2  				;0-15 pointer to the 16 values of 1700Hz lookup table
I_Osc:     		.space 2				;Current I 1700 Hz Oscillator sample
Q_Osc:     		.space 2               	;Current Q 1700 Hz Oscillator sample

I_Mixer_Out:	.space 2 				;I Mixer output
Q_Mixer_Out:	.space 2 				;Q Mixer output

OvlCounter:		.space 2				;Counts intervals to give overlaod LED flash delay
VOXAv:			.space 2				;Rolling average for VOX
VOXCounter:		.space 2				;Used for VOX timeout delays
Flags:			.space 2	
;..............................................................................;
;I2C_Count:		.space 2
;I2C_Length:		.space 2
;I2C_DataPtr:	.space 2

OffsetR:		.space 2			;DAC DC offsets
OffsetL:		.space 2
;*** SETUP uP ******

.text                            	;Start of Code section

__reset:						  	;Arrive here after uP reset

        mov 	#__SP_init, W15     ;Initalize the Stack Pointer
       	mov 	#__SP_init+#31, W0  ;Initialize the Stack Pointer Limit Register.
        mov 	W0, SPLIM			;
        nop                       	;Add NOP to follow SPLIM initialization

;---    							; 
        mov 	#38,w0        		;Set up clock. 7.37M div2 *40 div4 = ~35MIPS
        mov 	w0, PLLFBD
        mov 	#0x1000,w0
        mov 	w0, CLKDIV			

;---								;Set up clock to DAC: AClk set to 28.8kHz * 256

		mov		#0x0580,W0			;0X xx00 0100  1000 0000 PLL, DIV1	ACLK = PLL clock 
		mov		W0,ACLKCON			;ACLK USES FVCO !!! Datasheet wrong !!!
		
		mov		#0x00b4,W0			;Choose the type of binary data (fractional binary) sign+fract (0x7fff=+1:0x8000=-1)
		mov		W0,CORCON     	 	;& enable Program Space Visibility (Filter coeffs in ROM)

		call	FIR_Registers_Init	;Get pointers set up to sensible values
		call	ADC_Init			;12 bit. Auto sampling on ch0
		call	DAC_Init			;Turn on DACs, choose sample rate and select data format

       	bclr    TRISB, #LEDRed
		bclr	TRISB, #LEDGrn			;LED Outputs
		bclr	TRISB, #NVOX
		bset	CNPU1,#4			;Pullup on Weaver/phasing pin4
		bset	CNPU1,#5			;Pullup on LSB/USB pin5
		bset	CNPU1,#6			;Carrier mode (DC drive)

		mov		#0x4FAF,W0
		mov		W0,MODCON			;Select Y modulo addressing (circular buffers using W10 as pointer)

		clr		CycleCounter		;16 word samples every cycle of 1700Hz 
		mov 	#Q_OFFSETl,W1
		mov		W1,OffsetL			;Place in RAM just in case we ever want adjust on the fly
		mov 	#I_OFFSETr,W1
		mov		W1,OffsetR
		bset	IEC4,#DAC1RIE		;DAC interrupt enable

		clr		Flags
		clr		OvlCounter
		clr		VOXCounter
		clr		VOXAv
		
		bset	PORTB, #NVOX

;test:	bra		test
/*
   ----------------------------- Ready to run the program ------------------------------------

This is the main program loop. Rather than using a timer to define the sample rate it's driven by the DAC. 
We want the required sample rate to be able to sample 3400Hz. We choose 4 times Nyquist 28.8kHz which
means we oversample the audio to give a couple of extra bits.
To accept this sample rate the DAC needs a clock of 28.8k * 256 = 7.3728 MHz. We use a divider of 5 from the
uP clock speed giving an instruction rate of 36.8 MIPS) (NB If you work through my settings you'll find the
data sheet is wrong about the calculation of ACLK. ACLK is derived from the PLL VCO not the uP clock rate.)
Having done this the DAC then calls for samples when its buffers are empty at a rate of 28.8 kHz which drives
the program. The uP is interrupted and jumps to __DAC1RInterrupt: 

1. The DAC requests data as its buffers are empty
2. The interrupt is cleared and the last I/Q filter outputs are loaded into the DACs
3. Send the ADC sample to the BPF and process it in an FIR band pass filter
3a.  Check ADC value for overload threshold and flash red LED for [100ms]is overloaded
3b.  Check drive level is above a Min threshold, and illuminate green LED in real time when exceeded.  Gives a sort of continuous readout
3c.  Get an average amplitude and check against 'Schmitt type' thresholds.  Use timeout counter to give VOX
4. Look up the sine and cosine samples of the local 1800Hz oscillator (28800/16 = 1800Hz > 16 samples of a sine wave)
5. Pass the sin/cos samples to the mixers with the BPF output sample
6. Send the mixer outputs to the I and Q LPFs
6a. Add the calibration offsets
6b. Check if carrier link is in place, and send fixed levels if so,
7.   otherwise Send the LPF output samples to the DAC
8. Back to step 1 and repeat for ever.
   -----------------------------------------------------------------------------
*/

main:			
		bra		main				;Do stuff here - while waiting for interrupt.	
;===================================

;This is the transmit interrupt routine. Arrive here because the DAC needs more data - its buffer is empty.
__DAC1RInterrupt:					;Arrive here at 27.2kHz

		bclr	IFS4,#DAC1RIF		;clear DAC interrupt 	
		call	DAC					;Load DACs with calculations from last cycle (N-1).

;End of old cycle - beginning of new cycle

		call	ADC					;Get last sample from ADC conversion (N). Start sampling for (N+1)

		nop
		call	Mic_BPF_Filter		;Do FIR BP filtering on microphone channel
		call	OvlDet				;Check A/D input and flash Red LED if out of range
		call	CheckDrive			;Green LED on as drive exceeds Min threshold
		call	Osc_1700			;Generate I and Q oscillator samples
		call	Mixers				;Multiply Mic BPF output with I and Q Oscs
		call	I_LPF_Filter		;Filter mixer outputs
		call	Q_LPF_Filter

		retfie						;Return to main
;..............................................................................
;Subroutines
;..............................................................................



;-------------------- FIR Filters -----------------------
;We store the coefficients in X space (ROM). Use Y modular addressing to define the filter registers.
;Therefore we need W10 to be the Y pointer (as defined by the MAC instruction)
;Filters use:W0,5,6,8,10
I_LPF_Filter:	

		mov		#I_Delay,W10						;Set up start and end points of FIR data register
		mov		W10,YMODSRT							;From now on the uP does all the work of controlling the
		mov		#(I_Delay+402-1),W10				;circular buffer. (Addresses calculated as bytes. Each sample register is 2 bytes)
		mov		W10,YMODEND

		mov		#LPF_Kernel,W8						;Point to coeff, W8 used to point to X memory
		btss	PORTB,#RB0							;SELECT FILTER
		mov		#I_Phase,W8

		mov		I_Mixer_Out,W0						;Get current sample to add to I FIR register
		btss	PORTB,#0							;SELECT MIXER IF WEAVER
		mov		Mic_BPF_Out,W0


		mov		I_Ptr,W10							;Ready to write I FIR register at last sample
		mov     W0, [W10]            		       	;Move new sample into I register at ptr W10.



		clr     A, [W8]+=2, W5, [W10]+=2, W6        ;Clear MAC Accumulator A, load W5 with coeff pointed to by W8 and W6 oldest sample pointed by W10. Inc pointers
  													;for next Mult & Add (second MAC)
		repeat  #200                                ;Repeat following MAC instruction 201 times. Number of taps in filter.
		mac     W5*W6, A, [W8]+=2, W5, [W10]+=2, W6	;Multiply coeff by sample and add to Accumulator A. Inc pointers for next repeat
													;Move through delay line towards the beginning and forward through coefficients
		sac.r   A,#-2,W8								;Get accumulated coeffs(1-201) * samples. Shift left (equiv to mult by 2)
		btss	PORTB,#0
		sac.r   A,#0,W8
		
		mov		W8,I_LPF_Out						;Output sum to DAC
		mov     W10, I_Ptr		                    ;Update delay line pointer and save for next sample loop.	
		return
;-------------------------------
Q_LPF_Filter:
		mov		#Q_Delay,W10						;Set up start and end points of Q LPF FIR data register
		mov		W10,YMODSRT
		mov		#(Q_Delay+402-1),W10				;
		mov		W10,YMODEND

		mov		#LPF_Kernel,W8						;
		btss	PORTB,#RB0					;Switch 3rd - phasing
		mov		#Q_Phase,W8	

		mov		Q_Ptr,W10							;


		mov		Q_Mixer_Out,W0						;	
		btss	PORTB,#0
		mov		Mic_BPF_Out,W0

		mov     W0, [W10]  
 		       										;
		clr     A, [W8]+=2, W5, [W10]+=2, W6        ; 
		repeat  #200                                ;
		mac     W5*W6, A, [W8]+=2, W5, [W10]+=2, W6	; 
		sac.r   A,#-2,W8
		btss	PORTB,#0					;Account for gain change due to lower LPF normalised values
		sac.r   A,#0,W8
		mov		W8,Q_LPF_Out						;Output to DAC
		mov     W10, Q_Ptr		                    ;	
		return
;-----------------------------------
Mic_BPF_Filter:
		mov		#Mic_Delay,W10						;Set up start and end points of Mic BPF FIR data register
		mov		W10,YMODSRT
		mov		#(Mic_Delay+402-1),W10				; 201 tap FIR filter BP gets rid of ADC DC offsets 
		mov		W10,YMODEND							;

		mov		#BPF_Kernel,W8						;

		mov		Mic_Ptr,W10							;
		mov		Mic_ADC_Out,W0						;

		mov     W0, [W10]            		       	;

		clr     A, [W8]+=2, W5, [W10]+=2, W6        ;
		repeat  #200                                ;
		mac     W5*W6, A, [W8]+=2, W5, [W10]+=2, W6	;
		sac.r   A,#0,W8
		mov		W8,Mic_BPF_Out						;Output to mixers
		mov     W10, Mic_Ptr		                ;Update delay line pointer and save for next time.
		return
;-------------------------------------
ADC:	
		mov		ADC1BUF0,W0							;Save last sample
		mov		W0,Mic_ADC_Out
		mov		W0,ADC_Out
;last cycle
;----------
;new cycle
		bclr	AD1CON1,#SAMP  						;END sampling. START ADC conversion. We now have a complete cycle
													;for convertion to end and sampling to be automatically STARTED.
		return
;---------------------------------------
CheckDrive:				;Controls Green drive LED and VOX
		mov		Mic_BPF_Out, W0			;Drive based on output of L/P filter
 		mul.ss	W0, W0, W2				; Same ampliture as ADC, so 0 - 0x4000 in W3

		bclr	PORTB, #LEDGrn			;Grn LED on for levels above MiThresh, 
		mov		#MinThresh, W1			;Simple check on amplitude gives realistic change with level in
		sub		W3, W1, W0
		bra lt,	DriveLow		

		btss	Flags, #OvlFlag			;Green LED off if Overload
		bset	PORTB, #LEDGrn
DriveLow:
				;Now do a rolling average of [ampl]^2  values.  W3 is in range 0 - 16384.  KEEP W3 !!!!
		mov		VOXAv, W4				;Get existing value 
		mov		(#AVNo - #1), W1
		mul.uu	W4, W1, W0				;W0/1 = (AVNo - 1) * VOXAv 

		add		W0, W3, W0
		addc, 	#0, W1					;W0/1 = (AVNo - 1) * VOXAv + Latest

		mov		#AVNo, W4
		repeat	#17
		div.ud	W0, W4				;W0 = W0/1 / AvNo =   (VOXAv * (AVNo - 1) + Latest) / AVNo (ignore remainder in W1)
									;  Or VOXAv = (1 - x).VOXAv + x * Latest    where x = 1/AVNo
		mov		W0, VOXAv			;Update the rolling average
 			
;MinThresh = #40					;Use rolling average result to activate the green level setting LED
;		bclr	PORTB, #LEDGrn			;Grn LED on for levels above MinThresh, 
;		mov		#MinThresh, W1			
;		sub		W0, W1, W1
;		bra lt,	DriveLow2		

;		btss	Flags, #OvlFlag			;Green LED off if Overload
;		bset	PORTB, #LEDGrn
;DriveLow2:
		mov		#VOXUpperTh, W1		
		sub		W0, W1, W1			;W1 = VOXAv - Threshold
		bra lt, VOXLow
		bset	Flags, #VOXFlag		;Set Flag if > Upper Theshold
		goto	DoVOX
;.........	
VOXLow:
		mov		#VOXLowerTh, W1		;Clear flag if < Lower Threshold
		sub		W0, W1, W1			
		bra gt, DoVOX
		bclr	Flags, #VOXFlag
;..............
DoVOX:						;here every pass, act only on the flag
		inc		VOXCounter
		mov		#VOXDelay, W0			;Counts of 28800Hz 
		sub		VOXCounter, WREG	;W0 = Count - Threshold
		bra	ge,	VOXTimeout				

		btsc	Flags, #VOXFlag			;If flag is set, clear the counter to prevent timeout and set VOX on
		clr		VOXCounter
		btsc	Flags, #VOXFlag			;Ditto set VOX on if flag is set
		bclr	PORTB, #NVOX
		return
;..............
VOXTimeout:			;Here when counter overflows; the Flag has already been inherently cleared
		bset	PORTB, #NVOX
		bclr	Flags, #VOXFlag
		clr		VOXCounter
		return
;---------------------------------------
OvlDet:		;Flash RED LED when ADC exceeds MaxThresh.    Enters here every interrupt at 28800Hz
		btsc	Flags, #OvlFlag			;If the flag is set, just look for counter timeout
		goto	CheckOvlTimer			; and don't bother checking for anything else
		mov		Mic_ADC_Out, W0			;Raw ADC output is left justified signed. Square to give 0 - 0x40000000
 		mul.ss	W0, W0, W2				;  MS word in W3 ranges 0 - 0x4000 ( = 16384)

		mov		#MaxThresh, W1			;Overload Theshold at SQRT(number near maxsquared A/D output)
		sub		W3, W1, W0				;W0 = MS word ADC^2 - threshold
		bra le , CheckOvlTimer
		bset	PORTB, #LEDRed			;LED on for first instance of overload
		bclr	PORTB, #LEDGrn
		bset	Flags, #OvlFlag			;And set the flag
		return

;................
CheckOvlTimer:							;Until counter times, out, leave LED on and get out
		inc		OvlCounter
		mov		#OvlTimeout, W0			;Counts of 28800Hz 
		sub		OvlCounter, WREG	;W0 = Count - Threshold
		
		bra	ge,	LedOff				;Timed out, ready to test again
		return
;...................
LedOff:								;Clear overload situation and wait for next one
		bclr	PORTB, #LEDRed
		bclr	PORTB, #LEDGrn
		bclr	Flags, #OvlFlag
		clr		OvlCounter
		return
;---------------------------------------
;W0,W1,W2,W3
Osc_1700:

		mov		#SineGenTable,W0					;Point to sine lookup table
		mov		CycleCounter,W1						;Get current sine table position
		add		W0,W1,W0							;Make pointer to next sine sample
		mov		[W0],W2								;Move the sine sample to W2							
		mov		W2,I_Osc							;Load the sine sample
		mov		[W0+#32],W2							;Point to cosine table (32 bytes after sine table)
		mov		W2,Q_Osc							;Load cosine sample

		inc2	W1,W1								;Inc osc pointer (2 bytes = 1 word)
		and		#0x1f,W1							;Cycle pointer loops on 16th word (32nd byte)
		mov		W1,CycleCounter						;Save osc pointer for next program loop
		return
;----------------------------------------
;W0,1,2,3
Mixers:
		mov		Mic_BPF_Out,W0						;Get mic sample after BPF, signed multiply by osc sample
		mov		I_Osc,W1
		mul.ss	W0,W1,W2							;Answer in W3_W2. (throw away least significant word)						
		mov		W3,I_Mixer_Out						;(Take MS word of multiply) Transfer to output for LPF filtering

		mov		Mic_BPF_Out,W0						;Ditto for Q cosine channel
		mov		Q_Osc,W1
		mul.ss	W0,W1,W2
		mov		W3,Q_Mixer_Out
		return
;----------------------------------------
;W0
DAC:
		mov		Q_LPF_Out,W0						;Load data into DACs
		mov		I_LPF_Out,W1
		btss	PORTB,#1							;Swap I/Q channels for SSB select (Pin5)
		exch	W0,W1

		mov		OffsetR,W2							;Add DAC offset (balance) here
		add		W0,W2,W0					;There's overload potential here - and it does with offset in place
;~~~~~~~~~
		mov		#32767, W3		;Set the limiting value, -32768 or +32767 
		btss	SR, #N				; OVERFLOWING A SIGNED NUMBER FLIPS THE POLARITY
		mov		#-32768, W3			
		btsc	SR, #OV				;And if the result has overflowed, force in the limiting value
		mov		W3, W0			
;~~~~~~~~~
		btss	PORTB,#2		;If carrier mode, send fixed DC value to DAC
		mov		#Carrier,W0
		mov		W0,DAC1RDAT		

					
		mov		OffsetL,W2		
		add		W1,W2,W1
;~~~~~~~~~
		mov		#32767, W3		;Note the effect of overflowing a signed number
		btss	SR, #N
		mov		#-32768, W3
		btsc	SR, #OV
		mov		W3, W1
;~~~~~~~~~
		btss	PORTB,#2	
		mov		#Carrier,W1
		mov		W1,DAC1LDAT	

		return


ADC_Init:
        mov		# 0b1000011100000100,W0	;12bit, auto sample, manual conv, signed fractional output
		mov		W0,AD1CON1
		mov		# 0b1000000000000000,W0
		mov		W0,AD1CON2
		mov		# 0b0000000100000011,W0	
		mov		W0,AD1CON3
		mov		# 0b0000000000000000,W0
		mov		W0,AD1CON4
		mov		# 0b0000000000000000,W0
		mov		W0,AD1CHS123
		mov		#0xfffe,W0				;ADC on CH0
		mov		W0,AD1PCFGL
		clr		AD1CHS0
        return

DAC_Init:
		mov		#0x9100 + Dacdiv ,W0		;0b1x01xxx1 x0000100	Outputs ON, ACLK 
		mov		W0,	DAC1CON
		mov		#0x8680,W0		;0b1xxxxx1x 1xxxxx1x	Outputs ON,Interrupt on buffer empty
		mov		W0,DAC1STAT
		mov		#0x0000,W0
		mov		W0,DAC1DFLT		;OP  with no data = 0 
		mov		W0,DAC1LDAT		;DACs start with 0
		mov		W0,DAC1RDAT
		return

FIR_Registers_Init:
		mov		#I_Delay,W0		;Start writing at beginning of FIR data registers
		mov		W0,I_Ptr				
		mov		#Q_Delay,W0
		mov		W0,Q_Ptr
		mov		#Mic_Delay,W0
		mov		W0,Mic_Ptr
		return

;-------------------------------------------------------------------


.section .const,psv
;..............................................................................


;+/- 45deg and BPF used if simple phasing exciter selected.
I_Phase:
  .hword    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF 
  .hword    0xFFFE, 0xFFFD, 0xFFFD, 0xFFFE, 0x0000, 0x0002, 0x0001, 0xFFFF, 0xFFFB, 0xFFF7 
  .hword    0xFFF7, 0xFFFA, 0x0000, 0x0005, 0x0006, 0x0000, 0xFFF6, 0xFFEE, 0xFFEC, 0xFFF3 
  .hword    0x0000, 0x000C, 0x000F, 0x0006, 0xFFF3, 0xFFE2, 0xFFDC, 0xFFE8, 0x0000, 0x0018 
  .hword    0x0022, 0x0014, 0xFFF4, 0xFFD4, 0xFFC6, 0xFFD7, 0x0000, 0x002D, 0x0043, 0x0032 
  .hword    0x0000, 0xFFC7, 0xFFAA, 0xFFBF, 0x0000, 0x004D, 0x007A, 0x006A, 0x0020, 0xFFC3 
  .hword    0xFF8A, 0xFF9F, 0x0000, 0x007E, 0x00D3, 0x00CB, 0x0062, 0xFFD0, 0xFF69, 0xFF76 
  .hword    0x0000, 0x00C8, 0x0163, 0x0173, 0x00E5, 0x0000, 0xFF4A, 0xFF3D, 0x0000, 0x0141 
  .hword    0x025B, 0x02A9, 0x01ED, 0x007F, 0xFF30, 0xFEE1, 0x0000, 0x0234, 0x0474, 0x0580 
  .hword    0x0498, 0x0202, 0xFF1F, 0xFDE7, 0x0000, 0x05D9, 0x0E39, 0x1685, 0x1BB6, 0x1B91 
  .hword    0x159D, 0x0B6B, 0x0000, 0xF6AC, 0xF1C7, 0xF1E1, 0xF5B9, 0xFAF0, 0xFF1F, 0x00D5 
  .hword    0x0000, 0xFDB9, 0xFB8C, 0xFAAE, 0xFB73, 0xFD4C, 0xFF30, 0x0035, 0x0000, 0xFEE6 
  .hword    0xFDA5, 0xFCF9, 0xFD36, 0xFE2A, 0xFF4A, 0x0000, 0x0000, 0xFF66, 0xFE9D, 0xFE1D 
  .hword    0xFE29, 0xFEB3, 0xFF69, 0xFFEC, 0x0000, 0xFFAC, 0xFF2D, 0xFED0, 0xFEC9, 0xFF17 
  .hword    0xFF8A, 0xFFE7, 0x0000, 0xFFD4, 0xFF86, 0xFF46, 0xFF39, 0xFF63, 0xFFAA, 0xFFE9 
  .hword    0x0000, 0xFFEB, 0xFFBD, 0xFF94, 0xFF87, 0xFF9C, 0xFFC6, 0xFFEE, 0x0000, 0xFFF8 
  .hword    0xFFDE, 0xFFC5, 0xFFBB, 0xFFC5, 0xFFDC, 0xFFF3, 0x0000, 0xFFFE, 0xFFF1, 0xFFE3 
  .hword    0xFFDD, 0xFFE1, 0xFFEC, 0xFFF9, 0x0000, 0x0000, 0xFFFA, 0xFFF4, 0xFFF0, 0xFFF2 
  .hword    0xFFF7, 0xFFFC, 0x0000, 0x0000, 0xFFFF, 0xFFFC, 0xFFFB, 0xFFFB, 0xFFFD, 0xFFFF
  .hword    0x0000, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000 
  .hword    0x0000 
Q_Phase: ;Kernel Data
  .hword    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0x0000
  .hword    0x0000, 0xFFFF, 0xFFFD, 0xFFFB, 0xFFFB, 0xFFFC, 0xFFFF, 0x0000, 0x0000, 0xFFFC 
  .hword    0xFFF7, 0xFFF2, 0xFFF0, 0xFFF4, 0xFFFA, 0x0000, 0x0000, 0xFFF9, 0xFFEC, 0xFFE1 
  .hword    0xFFDD, 0xFFE3, 0xFFF1, 0xFFFE, 0x0000, 0xFFF3, 0xFFDC, 0xFFC5, 0xFFBB, 0xFFC5 
  .hword    0xFFDE, 0xFFF8, 0x0000, 0xFFEE, 0xFFC6, 0xFF9C, 0xFF87, 0xFF94, 0xFFBD, 0xFFEB
  .hword    0x0000, 0xFFE9, 0xFFAA, 0xFF63, 0xFF39, 0xFF46, 0xFF86, 0xFFD4, 0x0000, 0xFFE7
  .hword    0xFF8A, 0xFF17, 0xFEC9, 0xFED0, 0xFF2D, 0xFFAC, 0x0000, 0xFFEC, 0xFF69, 0xFEB3
  .hword    0xFE29, 0xFE1D, 0xFE9D, 0xFF66, 0x0000, 0x0000, 0xFF4A, 0xFE2A, 0xFD36, 0xFCF9 
  .hword    0xFDA5, 0xFEE6, 0x0000, 0x0035, 0xFF30, 0xFD4C, 0xFB73, 0xFAAE, 0xFB8C, 0xFDB9 
  .hword    0x0000, 0x00D5, 0xFF1F, 0xFAF0, 0xF5B9, 0xF1E1, 0xF1C7, 0xF6AC, 0x0000, 0x0B6B 
  .hword    0x159D, 0x1B91, 0x1BB6, 0x1685, 0x0E39, 0x05D9, 0x0000, 0xFDE7, 0xFF1F, 0x0202 
  .hword    0x0498, 0x0580, 0x0474, 0x0234, 0x0000, 0xFEE1, 0xFF30, 0x007F, 0x01ED, 0x02A9 
  .hword    0x025B, 0x0141, 0x0000, 0xFF3D, 0xFF4A, 0x0000, 0x00E5, 0x0173, 0x0163, 0x00C8 
  .hword    0x0000, 0xFF76, 0xFF69, 0xFFD0, 0x0062, 0x00CB, 0x00D3, 0x007E, 0x0000, 0xFF9F 
  .hword    0xFF8A, 0xFFC3, 0x0020, 0x006A, 0x007A, 0x004D, 0x0000, 0xFFBF, 0xFFAA, 0xFFC7 
  .hword    0x0000, 0x0032, 0x0043, 0x002D, 0x0000, 0xFFD7, 0xFFC6, 0xFFD4, 0xFFF4, 0x0014 
  .hword    0x0022, 0x0018, 0x0000, 0xFFE8, 0xFFDC, 0xFFE2, 0xFFF3, 0x0006, 0x000F, 0x000C 
  .hword    0x0000, 0xFFF3, 0xFFEC, 0xFFEE, 0xFFF6, 0x0000, 0x0006, 0x0005, 0x0000, 0xFFFA 
  .hword    0xFFF7, 0xFFF7, 0xFFFB, 0xFFFF, 0x0001, 0x0002, 0x0000, 0xFFFE, 0xFFFD, 0xFFFD 
  .hword    0xFFFE, 0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
  .hword    0x0000

BPF_Kernel: ;For microphone input

; Filter Kernel N=201 Fs=27200 F1=100 F2=3400
  .hword	0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF 
  .hword	0xFFFF, 0xFFFF, 0xFFFD, 0xFFFC, 0xFFFA, 0xFFFA, 0xFFFB, 0xFFFD, 0xFFFD, 0xFFFB 
  .hword	0xFFF7, 0xFFF2, 0xFFEE, 0xFFEE, 0xFFF2, 0xFFF6, 0xFFF8, 0xFFF4, 0xFFEA, 0xFFDE 
  .hword	0xFFD6, 0xFFD7, 0xFFE0, 0xFFEB, 0xFFEF, 0xFFE7, 0xFFD3, 0xFFBC, 0xFFAE, 0xFFB1 
  .hword	0xFFC3, 0xFFD9, 0xFFE3, 0xFFD4, 0xFFB0, 0xFF87, 0xFF6F, 0xFF77, 0xFF9A, 0xFFC3 
  .hword	0xFFD4, 0xFFBC, 0xFF80, 0xFF3C, 0xFF16, 0xFF26, 0xFF63, 0xFFAA, 0xFFCA, 0xFFA5 
  .hword	0xFF44, 0xFED9, 0xFE9E, 0xFEBC, 0xFF24, 0xFF9A, 0xFFD1, 0xFF99, 0xFF02, 0xFE5A 
  .hword	0xFE02, 0xFE35, 0xFEE1, 0xFFA3, 0x0000, 0xFFAD, 0xFEC1, 0xFDB7, 0xFD2D, 0xFD85 
  .hword	0xFEA3, 0xFFEB, 0x0091, 0x0011, 0xFE89, 0xFCC2, 0xFBCD, 0xFC67, 0xFE73, 0x00ED 
  .hword	0x0254, 0x0180, 0xFE63, 0xFA57, 0xF7AC, 0xF8A7, 0xFE59, 0x07E9, 0x12AE, 0x1B23 
  .hword	0x1E56, 0x1B23, 0x12AE, 0x07E9, 0xFE59, 0xF8A7, 0xF7AC, 0xFA57, 0xFE63, 0x0180 
  .hword	0x0254, 0x00ED, 0xFE73, 0xFC67, 0xFBCD, 0xFCC2, 0xFE89, 0x0011, 0x0091, 0xFFEB 
  .hword	0xFEA3, 0xFD85, 0xFD2D, 0xFDB7, 0xFEC1, 0xFFAD, 0x0000, 0xFFA3, 0xFEE1, 0xFE35 
  .hword	0xFE02, 0xFE5A, 0xFF02, 0xFF99, 0xFFD1, 0xFF9A, 0xFF24, 0xFEBC, 0xFE9E, 0xFED9 
  .hword	0xFF44, 0xFFA5, 0xFFCA, 0xFFAA, 0xFF63, 0xFF26, 0xFF16, 0xFF3C, 0xFF80, 0xFFBC 
  .hword	0xFFD4, 0xFFC3, 0xFF9A, 0xFF77, 0xFF6F, 0xFF87, 0xFFB0, 0xFFD4, 0xFFE3, 0xFFD9 
  .hword	0xFFC3, 0xFFB1, 0xFFAE, 0xFFBC, 0xFFD3, 0xFFE7, 0xFFEF, 0xFFEB, 0xFFE0, 0xFFD7 
  .hword	0xFFD6, 0xFFDE, 0xFFEA, 0xFFF4, 0xFFF8, 0xFFF6, 0xFFF2, 0xFFEE, 0xFFEE, 0xFFF2 
  .hword	0xFFF7, 0xFFFB, 0xFFFD, 0xFFFD, 0xFFFB, 0xFFFA, 0xFFFA, 0xFFFC, 0xFFFD, 0xFFFF 
  .hword	0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000 
  .hword	0x0000 

;......................... I and Q filters are identical so use the same kernel
 .section .const,psv

LPF_Kernel: ;LPF common to both Weaver channels


  ; LPF Filter kernel N=201 Fs=27200 F1=0 F2=1700

  .hword	0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF, 0xFFFF 
  .hword	0xFFFF, 0xFFFF, 0x0000, 0x0001, 0x0002, 0x0002, 0x0003, 0x0003, 0x0003, 0x0002
  .hword	0x0000, 0xFFFE, 0xFFFB, 0xFFF9, 0xFFF8, 0xFFF7, 0xFFF9, 0xFFFC, 0x0000, 0x0005
  .hword	0x000B, 0x000F, 0x0012, 0x0012, 0x000F, 0x0009, 0x0000, 0xFFF5, 0xFFEB, 0xFFE2 
  .hword	0xFFDD, 0xFFDD, 0xFFE3, 0xFFEF, 0x0000, 0x0013, 0x0026, 0x0036, 0x003E, 0x003D 
  .hword	0x0032, 0x001D, 0x0000, 0xFFDF, 0xFFBF, 0xFFA6, 0xFF98, 0xFF9A, 0xFFAD, 0xFFD0 
  .hword	0x0000, 0x0036, 0x006A, 0x0093, 0x00A9, 0x00A5, 0x0086, 0x004D, 0x0000, 0xFFAA 
  .hword	0xFF57, 0xFF15, 0xFEF2, 0xFEF8, 0xFF29, 0xFF84, 0x0000, 0x008C, 0x0113, 0x017F 
  .hword	0x01BB, 0x01B6, 0x0167, 0x00D1, 0x0000, 0xFF0D, 0xFE1A, 0xFD4C, 0xFCCC, 0xFCBE 
  .hword	0xFD3C, 0xFE53, 0x0000, 0x022F, 0x04BB, 0x0774, 0x0A1F, 0x0C80, 0x0E62, 0x0F96 
  .hword	0x1000, 0x0F96, 0x0E62, 0x0C80, 0x0A1F, 0x0774, 0x04BB, 0x022F, 0x0000, 0xFE53 
  .hword	0xFD3C, 0xFCBE, 0xFCCC, 0xFD4C, 0xFE1A, 0xFF0D, 0x0000, 0x00D1, 0x0167, 0x01B6 
  .hword	0x01BB, 0x017F, 0x0113, 0x008C, 0x0000, 0xFF84, 0xFF29, 0xFEF8, 0xFEF2, 0xFF15 
  .hword	0xFF57, 0xFFAA, 0x0000, 0x004D, 0x0086, 0x00A5, 0x00A9, 0x0093, 0x006A, 0x0036 
  .hword	0x0000, 0xFFD0, 0xFFAD, 0xFF9A, 0xFF98, 0xFFA6, 0xFFBF, 0xFFDF, 0x0000, 0x001D 
  .hword	0x0032, 0x003D, 0x003E, 0x0036, 0x0026, 0x0013, 0x0000, 0xFFEF, 0xFFE3, 0xFFDD 
  .hword	0xFFDD, 0xFFE2, 0xFFEB, 0xFFF5, 0x0000, 0x0009, 0x000F, 0x0012, 0x0012, 0x000F 
  .hword	0x000B, 0x0005, 0x0000, 0xFFFC, 0xFFF9, 0xFFF7, 0xFFF8, 0xFFF9, 0xFFFB, 0xFFFE 
  .hword	0x0000, 0x0002, 0x0003, 0x0003, 0x0003, 0x0002, 0x0002, 0x0001, 0x0000, 0xFFFF 
  .hword	0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 
  .hword	0x0000

.section .const,psv
SineGenTable:		;16 values of 1.7kHz sine/cos @ 16 samples/cycle
  .hword	0x0000,0x30FB,0x5A82,0x7641,0x7FFF,0x7641,0x5A82,0x30FB,0x0000,0xCF05,0xA57E,0x89BF,0x8000,0x89BF,0xA57E,0xCF05
.section .const,psv
CosineGenTable:	;	= sin table >> by 4 words						
  .hword	0x8000,0x89BF,0xA57E,0xCF05,0x0000,0x30FB,0x5A82,0x7641,0x7FFF,0x7641,0x5A82,0x30FB,0x0000,0xCF05,0xA57E,0x89BF


;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
















